declare class AbortError extends Error {
    constructor(msg?: string);
    get isAborted(): boolean;
}

declare type GeneratorExecutor<T> = (signal: AbortSignal) => Generator<any, T>;
declare type AbortInCoroutinesOptions = {
    signal?: AbortSignal;
    unhandledRejection?: boolean;
};
declare type OnFulfilled<T, R> = (value: T) => R | PromiseLike<R>;
declare type OnRejected<T = never> = (reason: any) => T | PromiseLike<T>;
declare class AbortInCoroutines<T> implements PromiseLike<T> {
    private _ctrl;
    private _promise;
    private _isAborted;
    constructor(gen: GeneratorExecutor<T>, { signal: optSig, unhandledRejection }?: AbortInCoroutinesOptions);
    then<TR1 = T, TR2 = never>(onfulfilled?: OnFulfilled<T, TR1> | null, onrejected?: OnRejected<TR2> | null): Promise<TR1 | TR2>;
    catch<TR = never>(onrejected?: OnRejected<TR> | null): Promise<T | TR>;
    finally(onfinally?: (() => void) | null): Promise<T>;
    get isAborted(): boolean;
    abort(): void;
}

declare function abortify<T = any, R = any>(combinator: (values: Iterable<T | PromiseLike<T>>) => PromiseLike<R>): (values: Iterable<T | PromiseLike<T>>) => AbortInCoroutines<R>;

declare function aico<T>(gen: GeneratorExecutor<T>, opts?: AbortInCoroutinesOptions): AbortInCoroutines<T>;

declare function cast<T>(p: PromiseLike<T>): Generator<PromiseLike<T>, T>;

declare const all: <T extends [] | readonly unknown[]>(values: T) => AbortInCoroutines<{ -readonly [P in keyof T]: Awaited<T[P]>; }>;
declare const race: <T extends [] | readonly unknown[]>(values: T) => AbortInCoroutines<Awaited<T[number]>>;
declare const any: <T extends [] | readonly unknown[]>(values: T) => AbortInCoroutines<Awaited<T[number]>>;
declare const allSettled: <T extends [] | readonly unknown[]>(values: T) => AbortInCoroutines<{ -readonly [P in keyof T]: PromiseSettledResult<Awaited<T[P]>>; }>;

export { AbortError, AbortInCoroutines, AbortInCoroutinesOptions, GeneratorExecutor, abortify, aico, all, allSettled, any, cast, race };
