// src/AbortError.ts
var AbortError = class extends Error {
  constructor(msg = "Aborted") {
    super(msg);
    this.name = "AbortError";
  }
  get isAborted() {
    return true;
  }
};

// src/AbortInCoroutines.ts
var AbortInCoroutines = class {
  _ctrl = null;
  _promise;
  _isAborted = false;
  constructor(gen, { signal: optSig, unhandledRejection = false } = {}) {
    const p = this._promise = new Promise((_resolve, _reject) => {
      let cleaners = [];
      function onAbort(sig, cb) {
        sig.addEventListener("abort", cb);
        cleaners.push(() => sig.removeEventListener("abort", cb));
      }
      const cleanup = () => {
        cleaners.forEach((f) => f());
        this._ctrl = cleaners = null;
      };
      const resolve = (val) => (_resolve(val), cleanup());
      const reject = (err) => (_reject(err), cleanup());
      const rejectWithAbort = (msg) => {
        this._isAborted = true;
        reject(new AbortError(msg));
      };
      if (optSig) {
        if (optSig.aborted) {
          return rejectWithAbort("`options.signal` is already aborted.");
        }
        onAbort(optSig, () => this.abort());
      }
      const { signal } = this._ctrl = new AbortController();
      const iter = gen(signal);
      let pRunning = null;
      let done = resolve;
      onAbort(signal, () => {
        if (pRunning && pRunning instanceof AbortInCoroutines)
          pRunning.abort();
        pRunning = null;
        done = (val) => {
          if (val === void 0) {
            rejectWithAbort();
          } else {
            resolve(val);
          }
        };
        const res = iter.return(void 0);
        if (res.done) {
          done(res.value);
        } else {
          handle(res);
        }
      });
      function nextIter(arg) {
        let res;
        try {
          res = iter.next(arg);
        } catch (err) {
          return reject(err);
        }
        handle(res);
      }
      function throwIter(arg) {
        let res;
        try {
          res = iter.throw(arg);
        } catch (err) {
          return reject(err);
        }
        handle(res);
      }
      function handle(res) {
        if (res.done) {
          return done(res.value);
        }
        if (isThenable(res.value)) {
          ;
          (pRunning = res.value).then((val) => pRunning === res.value && nextIter(val), (err) => pRunning === res.value && throwIter(err));
        } else {
          nextIter(res.value);
        }
      }
      nextIter();
    });
    if (!unhandledRejection) {
      p.catch(noop);
    }
  }
  then(onfulfilled, onrejected) {
    return this._promise.then(onfulfilled, onrejected);
  }
  catch(onrejected) {
    return this._promise.catch(onrejected);
  }
  finally(onfinally) {
    return this._promise.finally(onfinally);
  }
  get isAborted() {
    return this._isAborted;
  }
  abort() {
    var _a;
    (_a = this._ctrl) == null ? void 0 : _a.abort();
  }
};
function isThenable(p) {
  return p && typeof p.then === "function";
}
function noop() {
}

// src/abortify.ts
function abortify(combinator) {
  return (values) => new AbortInCoroutines(function* (signal) {
    let isErr = false;
    try {
      return yield combinator(values).then((v) => v, (err) => {
        isErr = true;
        throw err;
      });
    } finally {
      if (isErr || signal.aborted) {
        for (const p of values) {
          if (p instanceof AbortInCoroutines && !p.isAborted) {
            p.abort();
          }
        }
      }
    }
  });
}

// src/aico.ts
function aico(gen, opts) {
  return new AbortInCoroutines(gen, opts);
}

// src/cast.ts
function* cast(p) {
  return yield p;
}

// src/combinators.ts
var all = abortify(Promise.all.bind(Promise));
var race = abortify(Promise.race.bind(Promise));
var any = abortify(Promise.any.bind(Promise));
var allSettled = abortify(Promise.allSettled.bind(Promise));
export {
  AbortError,
  AbortInCoroutines,
  abortify,
  aico,
  all,
  allSettled,
  any,
  cast,
  race
};
